# Getting Started: Configuring Drupal to Apply Recipes

The following document describes the steps needed to configure a Composer based
Drupal installation to be ready to apply Drupal recipes to.

## CLI Composer.json Configuration

Run the following commands in order to update your composer.json file to be
ready for applying Drupal recipes.

```
# Configure Composer Patches.
composer config allow-plugins.cweagans/composer-patches true
composer require cweagans/composer-patches

# Add Drupal core patch for recipes to composer.json and update core.
composer config extra.patches --merge --json '{"drupal/core": {"Allow recipes to be applied": "https://git.drupalcode.org/project/distributions_recipes/-/raw/patch/recipe-10.2.x.patch"}}'
composer update drupal/core

# Configure the location recipes go when required.
composer config allow-plugins.oomphinc/composer-installers-extender true
composer config extra.installer-types --merge --json '["drupal-recipe"]'
composer config extra.installer-paths --merge --json '{"web/recipes/contrib/{$name}": ["type:drupal-recipe"]}'
composer require oomphinc/composer-installers-extender

# Requires Drupal Recipe Unpack.
composer config allow-plugins.ewcomposer/unpack true
composer config repo.recipe-unpack vcs https://github.com/woredeyonas/Drupal-Recipe-Unpack.git
composer require ewcomposer/unpack:dev-master
```

If you've done these steps, you can skip to *Preparing to Install Recipes*.

## Manual Composer.json Configuration

You can also make the changes needed to your composer.json manually. Folow these
 steps if you did not use the CLI.

### Composer Installers Extender

If your project does not have the composer-installers-extender plugin, follow
the instructions at https://github.com/oomphinc/composer-installers-extender and
 install it first.

This package lets you define new install-types for Composer so packages of
different types can be installed in different places in your project folder. If
this is not done, packages would be installed in the project's vendor folder.

In the repository's composer.json extra section, add `drupal-recipe` to
`installer-types`:

```
"extra": {
    "installer-types": [
        "drupal-library",
        "npm-asset",
        "bower-asset",
        "quicksilver-script",
        "drupal-recipe"
    ],
}
```

This tells composer to expect a new "type" defined in the recipe's composer.json
 file called `drupal-recipe`.

Also in the repository's composer.json extra section, add

"web/recipes/contrib/{$name}": ["type:drupal-recipe"]

```
    "installer-paths": {
        "web/core": ["type:drupal-core"],
        "web/libraries/{$name}": [
            "type:drupal-library",
            "type:bower-asset",
            "type:npm-asset"
        ],
        "web/modules/contrib/{$name}": ["type:drupal-module"],
        "web/profiles/contrib/{$name}": ["type:drupal-profile"],
        "web/themes/contrib/{$name}": ["type:drupal-theme"],
        "drush/contrib/{$name}": ["type:drupal-drush"],
        "web/private/scripts/quicksilver/{$name}/": ["type:quicksilver-script"],
        "web/recipes/contrib/{$name}": ["type:drupal-recipe"]
    },
```

This tells composer that when it encounters a drupal-recipe type package, to
install it in the web/recipes/contrib/ folder.

### Patch Drupal core for Recipes

If your project does not have the composer-patches plugin, follow the
instructions at https://github.com/cweagans/composer-patches and install it
first.

This plugin allows you to patch packages during composer install and update.

In the patches section of your site's composer.json apply the following patch.

NOTE: You may need to update the version of the patch based on your Drupal
version.

```
"patches": {
 "drupal/core": {
   "Allow recipes to be applied": "https://git.drupalcode.org/project/distributions_recipes/-/raw/patch/recipe-10.2.x.patch"
  }
},
```

NOTE: Please see the project page for the latest version patches.  At the time
of this writing, the following patches are available:

11.x

`curl https://git.drupalcode.org/project/distributions_recipes/-/raw/patch/recipe-11.x.patch | git apply`

10.3.x

`curl https://git.drupalcode.org/project/distributions_recipes/-/raw/patch/recipe-10.3.x.patch | git apply`

10.2.x

`curl https://git.drupalcode.org/project/distributions_recipes/-/raw/patch/recipe-10.2.x.patch | git apply`

10.1.x

`curl https://git.drupalcode.org/project/distributions_recipes/-/raw/patch/recipe-10.1.x.patch | git apply`

The patch is kept up to date with changes to the Recipes project, so once you
have it installed, it will automatically update with new updates.

Run `composer update drupal/core` to apply the patch.

### Add the Drupal Recipe Unpack Composer Plugin (optional)

The https://github.com/woredeyonas/Drupal-Recipe-Unpack composer plugin
can be run after you apply a recipe to add the package's from the recipe's
composer.json file to your main project's composer.json file.

The plugin is not on Packagist, so you need to install the repository in your
site's composer.json first.

```
{
    "type": "vcs",
    "url": "https://github.com/woredeyonas/Drupal-Recipe-Unpack.git"
}
```

When using Composer 2.2 or higher, Composer will ask for your permission to
allow this plugin to execute code.

You can add the following lines to your site's composer.json file's config >
allow-plugins section.

```
{
    "config": {
        "allow-plugins": {
            "ewcomposer/unpack": true,
        }
    }
}
```

Or run this command on the CLI:

`composer config allow-plugins.ewcomposer/unpack true`

## .gitignore Configuration

Because we are installing recipes using composer, you will want to update your
.gitignore file to exclude the path we added above to install contrib recipes
to:

`/web/recipes/contrib/`

You can still add custom/local recipes to the `/web/recipes/custom/` folder if
needed.

## Preparing to Install Recipes

Successfully installing a recipe requires the site be installed.  For minimal
configuration collisions it may be optimal to use the minimal install profile
while testing.

## Requiring and Applying Recipes

### From Packagist

If the recipe package is available from packagist, you will only need to require
 it.

`composer require kanopi/gin-admin-experience`

Verify that the composer.json and composer.lock files updated to include it.

### From a Git Repository

If the recipe package is in a public repository not available on Packagist,
first you need to add the location to your site's composer.json repositories
section:

```
{
 "type": "vcs",
 "url": "https://gitlab.com/kevinquillen/drupal-base"
}
```

Then require the package.

`composer require kevinquillen/drupal-base`

## Applying a recipe

Recipes are applied using core's PHP drupal script. In your CLI, CD into your
webroot (traditionally /web or /docroot depending on host), and run the
following command:

`php core/scripts/drupal recipe recipes/contrib/[recipe-name]`

Run `drush cr` to clear the cache, and verify on your site that the changes from
 the recipe were applied.

Follow your normal Git workflow to commit the changes.

### `recipe-apply` Docksal Command Helper

Here is a Docksal command called `recipe-apply` that makes it a bit easier by
cd-ing into the webroot, applying the recipe, and rebuilding the cache.  This
could be adapted to ddev and lando also.

```
#!/usr/bin/env bash

#: exec_target = cli

## Apply a Drupal Recipe that has been installed using composer.
##
## Usage: fin recipe-apply [name]
##

# Abort if anything fails
set -e

DOCROOT_PATH="${PROJECT_ROOT}/${DOCROOT}"

cd ${DOCROOT_PATH}

php core/scripts/drupal recipe recipes/contrib/"$@"

drush cr
```

### `drupal` DDEV Command Helper

Here is a ddev command called `drupal` that gives easier access to the drupal
script within your ddev web container.  To use add a file called drupal with the
following contents to either your global or project `.ddev/commands/web/`
directory.  Now you'll be able to run a command like `ddev drupal recipe recipes/contrib/[recipe-name]` in your project.

```
#!/bin/bash

## Description: run the Drupal core script with provided arguments
## Usage: drupal <arguments>
## Example: ddev drupal recipe recipes/contrib/drupal-base

cd ${DDEV_DOCROOT}

php core/scripts/drupal "$@"
```

## Unpacking a recipe

To "unpack", or move the recipe's package requirements to the site's
composer.json, run the following command:

`composer unpack [organization/package-name]`

After it completes, verify the packages have been added to the site's
composer.json

### `recipe-unpack` Docksal Command Helper

Here is a Docksal command called `recipe-unpack` that makes it a bit easier.
This could be adapted to ddev and lando also.

```
#!/usr/bin/env bash

#: exec_target = cli

## Unpack a Drupal Recipe to the site's composer.json
##
## Usage: fin recipe-unpack [organization/package-name]
##

# Abort if anything fails
set -e

composer unpack "$@"
```
