## Recipe types

Recipes are classified into types. Drawing on the work underway to convert core
install profiles into recipes, this page lists suggested types of recipes along
with notes about their usage.

In many cases, a particular type can be used in either of two contexts:
* _Single_. The recipe provides a single configuration item of the given type
  along with, in some cases, directly dependent configuration.
* _Multiple_. Generally the recipe should not provide configuration directly
  but instead apply other recipes that do.

### Recipe types and their usage

* 'Content field'
    * Single:
        * Naming pattern: `[type]_[id]` where `type` is a content type and
          `[id]` is the ID of a field minus any `field_` prefix.
        * Usage: adds a field to a content type and adds the field to relevant
          entity field and form displays.
        * Example: an `article_tags` recipe that adds a `tags` field to an
          `article` content type.
* 'Block type'
    * Single:
        * Naming pattern: `[type]_block` where `[type]` is the block content
          type.
        * Usage: provides a block type and associated configuration such as an
          entity view display.
        * Example: a `basic_block` recipe that provides a `basic` block type.
* 'Contact form'
    * Single:
        * Naming pattern: `[id]_contact_form` where `[id]` is the ID of a
          contact form.
        * Usage: provides a contact form.
        * Example: a `feedback_block` recipe that provides a `feedback` contact
          form.
* 'Content type'
    * Single:
        * Naming pattern: `[type]_content_type` where `[type]` is the content
          type.
        * Usage: provides a content type and associated configuration such as
          an entity view display.
        * Example: an `article_content_type` recipe that provides an `article`
          content type.
    * Multiple:
        * Naming pattern: `[descriptor]_content_types` where [descriptor]
          describes a group of content types.
        * Usage: applies other recipes that each provides a single content
          type.
        * Example: a `standard_content_types` recipe that applies the recipes
          that provide two commonly-used content types.
* 'Editor'
    * Single:
        * Naming pattern: `[id]_editor` where `[id]` is the ID of the editor.
        * Usage: provides an editor.
        * Example: a `basic_html_editor` recipe that provides a `basic_html`
          editor.
    * Multiple:
        * Naming pattern: `[descriptor]_editors` where [descriptor] describes a
          group of editors.
        * Usage: applies other recipes that each provides a single editor.
        * Example: a `standard_editors` recipe that applies the recipes that
          provide three commonly-used editors.
* 'Media type'
    * Single:
        * Naming pattern: `[type]_media` where `[type]` is the media type.
        * Usage: provides a media type and associated configuration such as an
          entity view display.
        * Example: an `audio_media` recipe that provides an `audio` media type.
    * Multiple:
        * Naming pattern: `[descriptor]_media_types` where [descriptor]
          describes a group of media types.
        * Usage: applies other recipes that each provides a single media type.
        * Example: a `standard_media_types` recipe that applies the recipes that
          provide several commonly-used media types.
* 'Responsive image'
    * Multiple:
        * Naming pattern: `[descriptor]_responsive_images` where `[descriptor]`
          describes a group of responsive image types.
        * Usage: provides two or more responsive image types. May also provide
          one or more image styles. Note: provisionally, in contrast with many
          other recipe types, multiple responsive image types are directly
          provided, rather than each being provided in its own recipe.
          Rationale: the responsive image styles are only usable as a set.
        * Example: a `standard_responsive_images` recipe that provides two
          responsive image types and related image styles.
* 'Site'
    * Single:
        * Naming pattern: none really, just `[name]` where `[name]` is a
          machine name for the site recipe.
        * Usage: the type 'Site' is a special case that allows such Drupal
          recipes to be listed on the installer.
        * Example: an `standard` recipe that provides a standard site with
          commonly used features pre-configured.
* 'Taxonomy'
    * Single:
        * Naming pattern: `[id]_taxonomy` where `[id]` is the ID of a taxonomy
          vocabulary.
        * Usage: provides a taxonomy vocabulary. May also provide a field
          storage for an entity reference field referencing taxonomy terms for
          use with the vocabulary.
        * Example: a `tags_taxonomy` recipe that provides a `tags` vocabulary.
* 'Text format'
    * Single:
        * Naming pattern: `[id]_format` where `[id]` is the ID of the format.
        * Usage: provides a text format.
        * Example: a `basic_html_format` recipe that provides a `basic_html` format.
* 'User role'
    * Single:
        * Naming pattern: `[id]_role` where `[id]` is the ID of the
          role.
        * Usage: provides a user role. Typically the role is provided without
          any permissions, as permissions will be added by other recipes via
          config actions.
        * Example: an `administrator_role` recipe that provides an
          `administrator` user role.
    * Multiple:
        * Naming pattern: `[descriptor]_roles` where [descriptor] describes a
          group of roles.
        * Usage: applies other recipes that each provides a single role.
        * Example: a `standard_roles` recipe that applies the recipes that
          provide two commonly-used roles.
* 'User type'
    * Single:
        * Naming pattern: `user`. Core only supports a single user type, so
          it's not foreseen that more than one recipe of this type is needed.
        * Usage: provides configuration for core's `user` user type.
        * Example: a `user` recipe.
* 'Workflow'
    * Single:
        * Naming pattern: `[id]_workflow` where `[id]` is the ID of the
          workflow.
        * Usage: provides a workflow.
        * Example: a `editorial_workflow` recipe that provides an `editorial`
          workflow.
